# Generated by roxygen2: do not edit by hand

export(IGMLE)
export(IGPIT)
export(IGScore)
export(testExponential)
export(testGLMGamma)
export(testGamma)
export(testLMNormal)
export(testNormal)
export(testYourModel)
import(statmod)
import(stats)
